"""Add receivers to alert definition

Revision ID: 314a5d69646a
Revises: 90c1158cc2ba
Create Date: 2025-08-13 20:25:53.585523

"""

from typing import Sequence, Union

import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "314a5d69646a"
down_revision: Union[str, None] = "90c1158cc2ba"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "alert_definition",
        sa.Column("receivers", postgresql.JSON(astext_type=sa.Text()), nullable=True),
    )
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("alert_definition", "receivers")
    # ### end Alembic commands ###
